from mashumaro.serializer.base.dict import DataClassDictMixin
from mashumaro.serializer.msgpack import DataClassMessagePackMixin
from mashumaro.serializer.yaml import DataClassYAMLMixin
from mashumaro.serializer.json import DataClassJSONMixin

from mashumaro.exceptions import MissingField
